package weakcache;

import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.net.URL;
import java.net.URLClassLoader;
import java.util.Arrays;
import java.util.Iterator;
import java.util.Scanner;

class RunJavaProcessAndCaptureException implements Runnable {
    private final String classToRun;

    public RunJavaProcessAndCaptureException(String classToRun) {
        this.classToRun = classToRun;
    }

    @Override
    public void run() {
        String classpath = getRuntimeClasspathJava6Compatible();
        String javaExec = getJavaExecutable();

        ProcessBuilder pb = new ProcessBuilder(javaExec, "-cp", classpath, classToRun);
        try {
            Process process = pb.start();
            process.waitFor();
            String errorOutput = streamToString(process.getErrorStream());

            if (errorOutput.contains("Exception") || errorOutput.contains("Error")) {
                throw new AssertionError("WeakCache has been reproduced: " + errorOutput);
            }
        } catch (Exception e) {
            throw new RuntimeException("Test execution error - this is NOT WeakCache issue", e);
        }
    }

    private String getRuntimeClasspathJava6Compatible() {
        URL[] urls = ((URLClassLoader) Thread.currentThread().getContextClassLoader()).getURLs();
        Iterator<URL> urlIterator = Arrays.asList(urls).iterator();

        StringBuilder sb = new StringBuilder(urlIterator.next().getFile());
        while(urlIterator.hasNext()) {
            sb.append(File.pathSeparator).append(urlIterator.next().getFile());
        }
        return sb.toString();
    }

    private String getJavaExecutable() {
        return System.getProperty("java.home") + File.separator + "bin" + File.separator + "java";
    }

    private String streamToString(InputStream stream) throws IOException {
        Scanner s = new Scanner(stream).useDelimiter("\\A");
        return s.hasNext() ? s.next() : "";
    }
}
