package weakcache;


import java.io.*;
import java.lang.reflect.InvocationHandler;
import java.lang.reflect.Method;
import java.lang.reflect.Proxy;
import java.util.concurrent.*;

public class WeakCacheHighLevelTest {
    public static final File serializableFile = new File(System.getProperty("java.io.tmpdir") + System.getProperty("file.separator") + "WeakCache.txt");

    public static void main(String[] args) throws ExecutionException, InterruptedException, IOException, ClassNotFoundException {
        if (serializableFile.exists() && ! serializableFile.delete()) {
            throw new IllegalStateException("Failed to delete " + serializableFile);
        }

        SayHi foo = (SayHi) Proxy.newProxyInstance(SayHi.class.getClassLoader(),
                new Class[] { SayHi.class },
                new SayHiProxy());

        Serializer.toFile(foo, serializableFile);

        TaskExecutor executor = new TaskExecutor(4);

        while (true) {
            executor.submitTaskNTimesAndCaptureException(new RunJavaProcessAndCaptureException(WeakCacheHighLevelTest.class.getName() + "$Runner"), 4);
        }
    }

    public static class Runner {
        public static void main(String[] args) throws IOException, ExecutionException, InterruptedException {
            new TaskExecutor(10).submitTaskNTimesAndCaptureException(new HighLevelTask(), 10);
        }

        static class HighLevelTask implements Runnable {
            public void run() {
                SayHi mySerializable = Serializer.fromFile(serializableFile);
                mySerializable.sayHi();
            }
        }
    }

    private static class Serializer implements Serializable {

        static <T> T fromFile(File f) {
            try {
                ObjectInputStream ois = new ObjectInputStream(new FileInputStream(f));
                @SuppressWarnings("unchecked")
                T o = (T) ois.readObject();
                ois.close();
                return o;
            } catch (Exception e) {
                throw new RuntimeException(e);
            }
        }

        static <T> void toFile(T t, File f) {
            try {
                FileOutputStream fout = new FileOutputStream(f);
                ObjectOutputStream oos = new ObjectOutputStream(fout);
                oos.writeObject(t);
                oos.close();
            } catch (Exception e) {
                throw new RuntimeException(e);
            }
        }
    }

    interface SayHi extends Serializable {
        String sayHi();
    }

    private static class SayHiImpl implements SayHi {
        public String sayHi() {
            return "hi";
        }
    }


    private static class SayHiProxy implements InvocationHandler, Serializable {
        private final SayHi sayHi = new SayHiImpl();

        public Object invoke(Object proxy, Method m, Object[] args)
                throws Throwable{
            return m.invoke(sayHi, args);
        }
    }
}
